/******************** (C) COPYRIGHT 2007 STMicroelectronics ********************
* File Name          : ST10f27x_can.c
* Author             : MCD Application Team
* Version            : V1.0
* Date               : 08/21/2007 
* Description        : This file provides all the ST10F27x CAN software 
*                     functions.
********************************************************************************
* THE PRESENT SOFTWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
* WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE TIME.
* AS A RESULT, STMICROELECTRONICS SHALL NOT BE HELD LIABLE FOR ANY DIRECT,
* INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING FROM THE
* CONTENT OF SUCH SOFTWARE AND/OR THE USE MADE BY CUSTOMERS OF THE CODING
* INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
*******************************************************************************/

/* Includes ------------------------------------------------------------------*/
#include "st10f27x_can.h"
/* Private typedef -----------------------------------------------------------*/
/* Private define ------------------------------------------------------------*/
/* Private macro -------------------------------------------------------------*/
/*----------------------------------------------------------------------------*/
/* Macro Name     : xxx_ID_MSK, xxx_ID_ARB                                    */
/* Description    : Form the Mask and Arbitration registers value to filter   */
/*                  a range of identifiers or a fixed identifier, for standard*/
/*                  and extended IDs                                          */
/*----------------------------------------------------------------------------*/
#define RANGE_ID_MSK(range_start, range_end)	(~((range_end) - (range_start)))
#define RANGE_ID_ARB(range_start, range_end)	((range_start) & (range_end))

#define FIXED_ID_MSK(id)	RANGE_ID_MSK((id), (id))
#define FIXED_ID_ARB(id)	RANGE_ID_ARB((id), (id))

#define STD_RANGE_ID_MSK(range_start, range_end)	((u16)((RANGE_ID_MSK((range_start), (range_end)) & 0x7FF) << 2))
#define STD_RANGE_ID_ARB(range_start, range_end)	((u16)(RANGE_ID_ARB((range_start), (range_end)) << 2))

#define STD_FIXED_ID_MSK(id)	((u16)((FIXED_ID_MSK(id) & 0x7FF) << 2))
#define STD_FIXED_ID_ARB(id)	((u16)(FIXED_ID_ARB(id) << 2))

#define EXT_RANGE_ID_MSK_L(range_start, range_end)	((u16)(RANGE_ID_MSK((range_start), (range_end))))
#define EXT_RANGE_ID_MSK_H(range_start, range_end)	((u16)(RANGE_ID_MSK((range_start), (range_end)) >> 16) & 0x1FFF)
#define EXT_RANGE_ID_ARB_L(range_start, range_end)	((u16)(RANGE_ID_ARB((range_start), (range_end))))
#define EXT_RANGE_ID_ARB_H(range_start, range_end)	((u16)(RANGE_ID_ARB((range_start), (range_end)) >> 16) & 0x1FFF)

#define EXT_FIXED_ID_MSK_L(id)	((u16)(FIXED_ID_MSK(id)))
#define EXT_FIXED_ID_MSK_H(id)	((u16)(FIXED_ID_MSK(id) >> 16 ) & 0x1FFF)
#define EXT_FIXED_ID_ARB_L(id)	((u16)(FIXED_ID_ARB(id)))
#define EXT_FIXED_ID_ARB_H(id)	((u16)(FIXED_ID_ARB(id) >> 16) & 0x1FFF)

/*----------------------------------------------------------------------------*/
/* Macro Name     : CAN_TIMING                                                */
/* Description    : Macro to format the timing register value from the        */
/*                  timing parameters                                         */
/*----------------------------------------------------------------------------*/
#define CAN_TIMING(tseg1, tseg2, sjw, brp)	((((tseg2-1) & 0x07) << 12) | (((tseg1-1) & 0x0F) << 8) | (((sjw-1) & 0x03) << 6) | ((brp-1) & 0x3F))
  
/* Private variables ---------------------------------------------------------*/
/* array of pre-defined timing parameters for standard bitrates calculated for*/
/* ST10 freuency = 40Mhz and the Prescaler /2 is activated at the CAN input   */   
u16 CanTimings[] = {        /* value   bitrate    NTQ  TSEG1  TSEG2  SJW  BRP */
  CAN_TIMING( 4, 3, 1, 25), /* 0x3AC4  100 kbit/s  8    4      3      1    24 */
  CAN_TIMING(11, 4, 4, 10), /* 0x3AC3  125 kbit/s 16    11     4      4    10 */
  CAN_TIMING( 4, 3, 3, 10), /* 0x2383  250 kbit/s  8    4      3      3    4  */
  CAN_TIMING( 4, 3, 1, 5),  /* 0x1C00  500 kbit/s  8    4      3      1    5  */
  CAN_TIMING( 4, 3, 1, 1)   /* 0x2300   1 Mbit/s   8    4      3      1    1  */
};

/* CAN IFx Interface defintion -----------------------------------------------*/
enum
{
    CAN_IF0,
	CAN_IF1,
	CAN_IF_BUSY
};

/* Private function prototypes -----------------------------------------------*/
static u16 CAN_GetFreeIF(CAN_TypeDef *CAN );

/* Private functions ---------------------------------------------------------*/ 
/*******************************************************************************
* Function Name  : CAN_GetFreeIF                                             
* Description    : Searchs the first free message interface, starting from 0.  
* Input          : CAN_TypeDef : pointer on the CAN interface; it can be CAN1
*                  or CAN2                                                       
* Output         : None                                                      
* Return         : A free message interface number (0 or 1) if found, else 2 
*******************************************************************************/
static u16 CAN_GetFreeIF(CAN_TypeDef *CAN)
{
     if ((CAN->sMsgObj[CAN_IF0].CRR & CAN_CRR_BUSY) == 0)
       return CAN_IF0;

    else if ((CAN->sMsgObj[CAN_IF1].CRR & CAN_CRR_BUSY) == 0)
       return CAN_IF1;

	else
	 return CAN_IF_BUSY;
}

/*******************************************************************************
* Function Name  : CAN_EnterInitMode                                         
* Description    : Switchs the CAN into initialization mode. This function must
*                  be used in conjunction with CAN_LeaveInitMode().                 
* Input          : - CAN_TypeDef : pointer on the CAN interface; it can be CAN1
*                  or CAN2 
*                  - any binary value formed from the CAN_CR_xxx defines      
* Output         : None                                                      
* Return         : None                                                          
*******************************************************************************/
void CAN_EnterInitMode(CAN_TypeDef *CAN, u16 Mask)
{
  CAN->CR =  Mask | CAN_CR_INIT ;
  /* reset the status*/
  CAN->SR = 0;				   
}

/*******************************************************************************
* Function Name  : CAN_LeaveInitMode                                         
* Description    : Leaves the initialization mode (switch into normal mode).
*                  This function must be used in conjunction with CAN_EnterInitMode().  
* Input          : CAN_TypeDef : pointer on the CAN interface; it can be CAN1
*                  or CAN2                                                      
* Output         : None                                                      
* Return         : None                                                      
*******************************************************************************/
void CAN_LeaveInitMode(CAN_TypeDef *CAN)
{
  CAN->CR &= (u16)(~( CAN_CR_CCE | CAN_CR_INIT));
}

/*******************************************************************************
* Function Name  : CAN_EnterTestMode                                         
* Description    : Switchs the CAN into test mode. This function must be used in
*                  conjunction with CAN_LeaveTestMode().                            
* Input          : - TestMask: specifies the configuration in test modes.
*                  - CAN_TypeDef : pointer on the CAN interface; it can be CAN1
*                  or CAN2      
* Output         : None                                                      
* Return         : None                                                            
*******************************************************************************/
void CAN_EnterTestMode(CAN_TypeDef *CAN, u16 TestMask)
{
  CAN->CR |= CAN_CR_TEST;
  CAN->TESTR |= TestMask;
}

/*******************************************************************************
* Function Name  : CAN_LeaveTestMode                                         
* Description    : Leaves the current test mode (switch into normal mode).
*                  This function must be used in conjunction with CAN_EnterTestMode().    
* Input          : CAN_TypeDef : pointer on the CAN interface; it can be CAN1
*                  or CAN2                                                       
* Output         : None                                                      
* Return         : None                                                      
*******************************************************************************/
void CAN_LeaveTestMode(CAN_TypeDef *CAN)
{
  CAN->CR |= CAN_CR_TEST;
  CAN->TESTR &= (u16)(~(CAN_TESTR_LBACK | CAN_TESTR_SILENT | CAN_TESTR_BASIC));
  CAN->CR &= (u16)(~CAN_CR_TEST);
}

/*******************************************************************************
* Function Name  : CAN_SetBitrate                                            
* Description    : Sets up a standard CAN bitrate for a ST10 frequency of 40 Mhz 
*                  and a prescaler 2 is activated in XMISC register                               
* Input          : One of the following 
*                  - CAN_BITRATE_100K,
*                  - CAN_BITRATE_125K,
*                  - CAN_BITRATE_250K,
*                  - CAN_BITRATE_500K,
*                  - CAN_BITRATE_1M      
* Output         : None                                                      
* Return         : None                                                                         
*******************************************************************************/
void CAN_SetBitrate(CAN_TypeDef *CAN, u16 bitrate)
{
    /* write the predefined timing value */
    CAN->BTR = CanTimings[bitrate];   
     		           
    /* clear the Extended Baud Rate Prescaler */
    CAN->BRPR = 0;
}

/*******************************************************************************
* Function Name  : CAN_SetTiming                                             
* Description    : Sets up the CAN timing with specific parameters             
* Input          : - tseg1: specifies Time Segment before the sample point.
*                    This parameter must be a number between 1 and 16.       
*                  - tseg2: Time Segment after the sample point. This parameter 
*                    must be a number between 1 and 8.        
*                  - sjw: Synchronisation Jump Width. This parameter must be                 
*                     a number between 1 and 4.
*                  - brp: Baud Rate Prescaler. This parameter must be a number
*                    between 1 and 1024. 
*                  - CAN_TypeDef : pointer on the CAN interface; it can be CAN1
*                  or CAN2                                         
* Output         : None                                                      
* Return         : None                                                                        
*******************************************************************************/
void CAN_SetTiming(CAN_TypeDef *CAN, u16 tseg1, u16 tseg2, u16 sjw, u16 brp)
{
  CAN->BTR = CAN_TIMING(tseg1, tseg2, sjw, brp);
  CAN->BRPR = ((brp-1) >> 6) & 0x0F;
}

/*******************************************************************************
* Function Name  : CAN_Init                                                  
* Description    : Initializes the CAN peripheral according to the entered 
*                  parameters.                                            
* Input          : - mask : it can be a combination of CAN_CR_xxx defines   
*                  - CAN_TypeDef : pointer on the CAN interface; it can be CAN1
*                  or CAN2 
*                  - Bitrate: one of the CAN_BITRATE_xxx defines
* Output         : None                                                      
* Return         : None                                                      
*******************************************************************************/
void CAN_Init(CAN_TypeDef *CAN , u16 Mask, u16 Bitrate)
{
  /* Set the CCE bit in the CAN control register to acceed the BTR register */
  CAN_EnterInitMode(CAN, CAN_CR_CCE | Mask);

   CAN_SetBitrate(CAN, Bitrate);
  
  /* Leave the initialization mode */
   CAN_LeaveInitMode(CAN); 

  /* Leave the Test mode */
  CAN_LeaveTestMode( CAN );
}

/*******************************************************************************
* Function Name  : CAN_SetUnusedMsgObj                                       
* Description    : Configures the message object as unused (invalid)                   
* Input          :- msgobj: specifies the Message object number, from 0 to 31.
*                 - CAN_TypeDef : pointer on the CAN interface; it can be CAN1
*                  or CAN2                       
* Output         : None                                                      
* Return         : An ErrorStatus enumuration value:
*                         - SUCCESS: Interface to treat the message
*                         - ERROR: No interface to treat the message
*******************************************************************************/
ErrorStatus CAN_SetUnusedMsgObj(CAN_TypeDef *CAN, u16 msgobj)
{
  u16 msg_if=0;

  if ((msg_if = CAN_GetFreeIF(CAN)) == CAN_IF_BUSY)
    return ERROR;
 
  /* fill the command mask register to write into arb, mask, data and control 
  registers */
  CAN->sMsgObj[msg_if].CMR = CAN_CMR_WRRD
                           | CAN_CMR_MASK
                           | CAN_CMR_ARB
                           | CAN_CMR_CONTROL
                           | CAN_CMR_DATAA
                           | CAN_CMR_DATAB;

  CAN->sMsgObj[msg_if].M1R = 0;
  CAN->sMsgObj[msg_if].M2R = 0;

  CAN->sMsgObj[msg_if].A1R = 0;
  CAN->sMsgObj[msg_if].A2R = 0;

  CAN->sMsgObj[msg_if].MCR = 0;

  CAN->sMsgObj[msg_if].DA1R = 0;
  CAN->sMsgObj[msg_if].DA2R = 0;
  CAN->sMsgObj[msg_if].DB1R = 0;
  CAN->sMsgObj[msg_if].DB2R = 0;

 CAN->sMsgObj[msg_if].CRR = 1 + msgobj; 
 return SUCCESS;
}

/*******************************************************************************
* Function Name  : CAN_SetUnusedAllMsgObj				      
* Description    : Configures all the message objects as unused( invalid).               
* Input          : - CAN_TypeDef : pointer on the CAN interface; it can be CAN1
*                  or CAN2                                                        
* Output         : None                                                      
* Return         : An ErrorStatus enumuration value:
*                  - ERROR if one of the CAN message object is not set to unsed
*                  - SUCCESS else                                                      
*******************************************************************************/
ErrorStatus CAN_SetUnusedAllMsgObj(CAN_TypeDef *CAN)
{
  u16 msg=0;
  for (msg = 0; msg < 32; msg++)
  {
    if( CAN_SetUnusedMsgObj(CAN, msg) == ERROR)
    return ERROR;
  }
  return SUCCESS;
}

/*******************************************************************************
* Function Name  : CAN_ReleaseMessage					      
* Description    : Releases the message object                                
* Input          : - msgobj: specifies the Message object number, from 0 to 31.
*                  - CAN_TypeDef : pointer on the CAN interface; it can be CAN1
*                  or CAN2                      
* Output         : None                                                      
* Return         : An ErrorStatus enumuration value:
*                         - SUCCESS: Interface to treat the message
*                         - ERROR: No interface to treat the message
*******************************************************************************/
ErrorStatus CAN_ReleaseMessage(CAN_TypeDef *CAN, u16 msgobj)
{
  u16 msg_if=0;

   if ((msg_if = CAN_GetFreeIF(CAN)) == CAN_IF_BUSY)
    return ERROR;
  
  /* clear the interrupt and transmit request bits in the command mask register */
  CAN->sMsgObj[msg_if].CMR = CAN_CMR_CLRINTPND | CAN_CMR_TXRQSTNEWDAT;

  /* Write into the command request register */	
  CAN->sMsgObj[msg_if].CRR = 1 + msgobj;

  return SUCCESS;
}

/*******************************************************************************
* Function Name  : CAN_SetTxMsgObj                                           
* Description    : Configures the message object as TX.                        
* Input          : - msgobj: specifies the Message object number, from 0 to 31.                      
*                  - idType: specifies the identifier type of the frames that
*                    will be transmitted using this message object.
*                    This parameter can be one of the following values:
*                          - CAN_STD_ID (standard ID, 11-bit)
*                          - CAN_EXT_ID (extended ID, 29-bit)   
*                 - RemoteEN : specifies the remote enable bit in the msg control
*                   register. If set the msg object answers automatically remote
*                   frames with matching ID
*                 The transmit interrupt is enabled in this function
*                 The mask is set into the value 0x3FF or to 0x1FFFFFFF
*                  - CAN_TypeDef : pointer on the CAN interface; it can be CAN1
*                  or CAN2                                   
* Output         : None                                                      
* Return         : An ErrorStatus enumuration value:
*                         - SUCCESS: Interface to treat the message
*                         - ERROR: No interface to treat the message
*******************************************************************************/
ErrorStatus CAN_SetTxMsgObj(CAN_TypeDef *CAN, u16 msgobj, u16 idType, FunctionalState RemoteEN)
{
  u16 msg_if=0;

  if ((msg_if = CAN_GetFreeIF(CAN)) == CAN_IF_BUSY)
    return ERROR;
  
  CAN->sMsgObj[msg_if].CMR = CAN_CMR_WRRD
                           | CAN_CMR_MASK
                           | CAN_CMR_ARB
                           | CAN_CMR_CONTROL
                           | CAN_CMR_DATAA
                           | CAN_CMR_DATAB;

  CAN->sMsgObj[msg_if].M1R = 0xFFFF;
  CAN->sMsgObj[msg_if].A1R = 0;

  if (idType == CAN_STD_ID)
  {
    CAN->sMsgObj[msg_if].M2R = CAN_M2R_MDIR | 0x1FFF;
    CAN->sMsgObj[msg_if].A2R = CAN_A2R_MSGVAL | CAN_A2R_DIR;
  }
  else
  {
    CAN->sMsgObj[msg_if].M2R = CAN_M2R_MDIR | CAN_M2R_MXTD | 0x1FFF;
    CAN->sMsgObj[msg_if].A2R = CAN_A2R_MSGVAL | CAN_A2R_DIR | CAN_A2R_XTD;
  }

  CAN->sMsgObj[msg_if].MCR =  CAN_MCR_TXIE | ( RemoteEN ? CAN_MCR_UMASK : 0) | CAN_MCR_EOB |
                              ( RemoteEN ?  CAN_MCR_RMTEN : 0);

  CAN->sMsgObj[msg_if].DA1R = 0;
  CAN->sMsgObj[msg_if].DA2R = 0;
  CAN->sMsgObj[msg_if].DB1R = 0;
  CAN->sMsgObj[msg_if].DB2R = 0;

  CAN->sMsgObj[msg_if].CRR = 1 + msgobj;

  return SUCCESS;
}	

/*******************************************************************************
* Function Name  : CAN_SetRxMsgObj                                           
* Description    : Configures the message object as RX.                        
* Input          : - msgobj: specifies the Message object number, from 0 to 31.                    
*                  - idType: specifies the identifier type of the frames that
*                    will be transmitted using this message object.
*                    This parameter can be one of the following values:
*                          - CAN_STD_ID (standard ID, 11-bit)
*                          - CAN_EXT_ID (extended ID, 29-bit)                               
*                  - idLow: specifies the low part of the identifier range used      
*                    for acceptance filtering.
*                  - idHigh: specifies the high part of the identifier range    
*                    used for acceptance filtering.                 
*                  - singleOrFifoLast: specifies the end-of-buffer indicator.
*                    This parameter can be one of the following values:
*                          - TRUE: for a single receive object or a FIFO receive
*                            object that is the last one of the FIFO. 
*                          - FALSE: for a FIFO receive object that is not the 
*                            last one. 
*                The receive interrupt is enabled in this function.
*                  - CAN_TypeDef : pointer on the CAN interface; it can be CAN1
*                  or CAN2      
* Output         : None                                                      
* Return         : An ErrorStatus enumuration value:
*                         - SUCCESS: Interface to treat the message
*                         - ERROR: No interface to treat the message
*******************************************************************************/
ErrorStatus CAN_SetRxMsgObj(CAN_TypeDef *CAN, u16 msgobj, u16 idType, u32 idLow, u32 idHigh, 
                             bool singleOrFifoLast) 
{
  u16 msg_if=0;

   if ((msg_if = CAN_GetFreeIF(CAN)) == CAN_IF_BUSY)
    return ERROR;

  CAN->sMsgObj[msg_if].CMR = CAN_CMR_WRRD
                           | CAN_CMR_MASK
                           | CAN_CMR_ARB
                           | CAN_CMR_CONTROL
                           | CAN_CMR_DATAA
                           | CAN_CMR_DATAB;

  if (idType == CAN_STD_ID)
  {
    CAN->sMsgObj[msg_if].M1R = 0xFFFF;
    CAN->sMsgObj[msg_if].M2R = CAN_M2R_MDIR | CAN_M2R_MXTD | STD_RANGE_ID_MSK(idLow, idHigh);

    CAN->sMsgObj[msg_if].A1R = 0;
    CAN->sMsgObj[msg_if].A2R = CAN_A2R_MSGVAL | STD_RANGE_ID_ARB(idLow, idHigh);
  }
  else
  {
    CAN->sMsgObj[msg_if].M1R = EXT_RANGE_ID_MSK_L(idLow, idHigh);
    CAN->sMsgObj[msg_if].M2R = CAN_M2R_MDIR |CAN_M2R_MXTD | EXT_RANGE_ID_MSK_H(idLow, idHigh);

    CAN->sMsgObj[msg_if].A1R = EXT_RANGE_ID_ARB_L(idLow, idHigh);
    CAN->sMsgObj[msg_if].A2R = CAN_A2R_MSGVAL | CAN_A2R_XTD | EXT_RANGE_ID_ARB_H(idLow, idHigh);
	}

  CAN->sMsgObj[msg_if].MCR = CAN_MCR_UMASK | CAN_MCR_RXIE  | (singleOrFifoLast ? CAN_MCR_EOB : 0);

  CAN->sMsgObj[msg_if].DA1R = 0;
  CAN->sMsgObj[msg_if].DA2R = 0;
  CAN->sMsgObj[msg_if].DB1R = 0;
  CAN->sMsgObj[msg_if].DB2R = 0;

  CAN->sMsgObj[msg_if].CRR = 1 + msgobj;

  return SUCCESS;
}	

/*******************************************************************************
* Function Name  : ErrorStatus CAN_UpdateMsgObj                                            
* Description    : This function updates the data of a message object without 
*				   changing the remaining fields of the message control register
*                  This function does not update the ID type of a message object 				                
* Input          : - Pointer on a CAN_TypdeDef structure 
*                  - msgobj: number of the message object that should be updated
*                  - Pointer on a sructure of canmsg type  
*				   - Data length 						          
* Output         : None                                                      
* Return         : An ErrorStatus enumuration value:
*                         - SUCCESS: Interface to treat the message
*                         - ERROR: No interface to treat the message
*******************************************************************************/
ErrorStatus CAN_UpdateMsgObj( CAN_TypeDef* CAN, u16 msgobj, canmsg* pCanMsg)
{
	 	  	  	       
    if (CAN->sMsgObj[0].CRR & CAN_CRR_BUSY) 
    return ERROR;

	/* Write into the CMR to read the arbitration, control fields  */ 
	CAN->sMsgObj[0].CMR = CAN_CMR_CONTROL | CAN_CMR_ARB;

    /* start the transfer by writing into the command request register */
	CAN->sMsgObj[0].CRR = 1 + msgobj;

	/* wait until the transfer is complete */ 
	while( CAN->sMsgObj[0].CRR & CAN_CRR_BUSY); 	
	 
	 /* Test if the message object Id configuration and and the new Id are the same */
	 /* return ERROR if this is not the case */
	 if( ( pCanMsg->IdType << 14) != (CAN->sMsgObj[0].A2R & CAN_A2R_XTD))
	 return 0;
	    
	 /* Update the arbitration field */
	 if ((CAN->sMsgObj[0].A2R & CAN_A2R_XTD) == 0)
  	 {
       /* standard ID */
       CAN->sMsgObj[0].A1R = 0;
       CAN->sMsgObj[0].A2R = (CAN->sMsgObj[0].A2R & 0xE000) | STD_FIXED_ID_ARB(pCanMsg->Id);
      }
     else
      {
       /* extended ID */
	   CAN->sMsgObj[0].A1R = EXT_FIXED_ID_ARB_L(pCanMsg->Id);
	   CAN->sMsgObj[0].A2R = (CAN->sMsgObj[0].A2R & 0xE000) | EXT_FIXED_ID_ARB_H(pCanMsg->Id);
	  }

	 /* Update the control field */
	 CAN->sMsgObj[0].MCR = ( CAN->sMsgObj[0].MCR & 0xFFF0 )  | CAN_MCR_NEWDAT | (pCanMsg->Dlc);

	  /* Update the Data field */ 
      CAN->sMsgObj[0].DA1R = (pCanMsg->Data[1] << 8) | pCanMsg->Data[0];
	  CAN->sMsgObj[0].DA2R = (pCanMsg->Data[3] << 8) | pCanMsg->Data[2];
	  CAN->sMsgObj[0].DB1R = (pCanMsg->Data[5] << 8) | pCanMsg->Data[4];
	  CAN->sMsgObj[0].DB2R = (pCanMsg->Data[7] << 8) | pCanMsg->Data[6]; 

       /* Write into the CMR to update the data, arbitration and control fields*/ 
	   CAN->sMsgObj[0].CMR = CAN_CMR_WRRD
                                | CAN_CMR_ARB
                                | CAN_CMR_CONTROL
                                | CAN_CMR_DATAA
                                | CAN_CMR_DATAB;

	 /* Write the message object number in the command request register */   
	  CAN->sMsgObj[0].CRR = 1 + msgobj;

	  while( CAN->sMsgObj[0].CRR & CAN_CRR_BUSY );  
	 
     return SUCCESS;   
}

/*******************************************************************************
* Function Name  : CAN_TransmitRequest                                            
* Description    : This function requests the transmission of a message object 			                
* Input          : - Pointer on a CAN_TypdeDef structure 
*                  - msgobj: number of the message object that should be 
*                    transmitted  
* Output         : None                                                      
* Return         : An ErrorStatus enumuration value:
*******************************************************************************/
ErrorStatus CAN_TransmitRequest( CAN_TypeDef *CAN, u16 msgobj )
{
  u16 msg_if;

    if ((msg_if = CAN_GetFreeIF(CAN)) == CAN_IF_BUSY) 
     return ERROR;

     /* Set the transmit request in the command mask register */ 
     CAN->sMsgObj[msg_if].CMR = CAN_CMR_WRRD | CAN_CMR_TXRQSTNEWDAT;

   /* Write the message object number in the command request register */
   CAN->sMsgObj[msg_if].CRR = 1 + msgobj;

   return SUCCESS;
}


/*******************************************************************************
* Function Name  : CAN_SendMessage                                           
* Description    : Update the id and the data of the message object passed in 
*                  parameter and starts its transmission                            
* Input          : - Pointer on a CAN_TypdeDef structure
*                : - msgobj: specifies the Message object number, from 0 to 31.                    
*                : - pCanMsg: pointer to the message structure containing data     
*                    to transmit.
* Output         : None                                                      
* Return         : An ErrorStatus enumuration value:
*                         - SUCCESS: Transmission OK
*                         - ERROR: No transmission
*******************************************************************************/
ErrorStatus CAN_SendMessage(CAN_TypeDef *CAN, u16 msgobj, canmsg* pCanMsg)
{
  if( CAN_UpdateMsgObj( CAN, msgobj, pCanMsg) == ERROR) return ERROR;

	/* clear the transmit OK flag in the CAN status register */
  CAN->SR &= (u16)(~CAN_SR_TXOK);

  return( CAN_TransmitRequest( CAN, msgobj) );
}

/*******************************************************************************
* Function Name  : CAN_ReceiveMessage                                        
* Description    : Gets the message, if received.
* Input          : - Pointer on a CAN_TypdeDef structure
*                  - msgobj: specifies the Message object number, from 0 to 31.                     
*                  - release: specifies the message release indicator.
*                    This parameter can be one of the following values:
*                          - TRUE: the message object is released when getting  
*                            the data.
*                          - FALSE: the message object is not released.
*                  - pCanMsg: pointer to the message structure where received   
*                    data is copied.
* Output         : None                                                      
* Return         : An ErrorStatus enumuration value:
*                         - SUCCESS: Reception OK
*                         - ERROR: No message pending
*******************************************************************************/
ErrorStatus CAN_ReceiveMessage(CAN_TypeDef *CAN, u16 msgobj, bool release, canmsg* pCanMsg)
{
  u16 msgif;
  vu32 test = 0;

  if (!CAN_GetMsgReceiveStatus(CAN, msgobj))
    return ERROR;
  
  CAN->SR &= (u16)(~CAN_SR_RXOK);

  /* Get Free Interface  */ 
  if ((msgif = CAN_GetFreeIF(CAN)) == CAN_IF_BUSY) return ERROR;

  /* read the message contents */
  CAN->sMsgObj[msgif].CMR = CAN_CMR_MASK
                      | CAN_CMR_ARB
                      | CAN_CMR_CONTROL
                      | CAN_CMR_CLRINTPND
                      | (release ? CAN_CMR_TXRQSTNEWDAT : 0)
                      | CAN_CMR_DATAA
                      | CAN_CMR_DATAB;

  CAN->sMsgObj[msgif].CRR = 1 + msgobj;

  /* wait until the message object is read by the interface */
  while( CAN->sMsgObj[msgif].CRR & CAN_CRR_BUSY);
      
  if ((CAN->sMsgObj[msgif].A2R & CAN_A2R_XTD) == 0)
  {
    /* standard ID */
    pCanMsg->IdType = CAN_STD_ID;
    pCanMsg->Id = (CAN->sMsgObj[msgif].A2R >> 2) & 0x07FF;
  }
  else
  {
    /* extended ID */
    pCanMsg->IdType = CAN_EXT_ID;
	pCanMsg->Id  = CAN->sMsgObj[msgif].A1R | ((u32)(CAN->sMsgObj[msgif].A2R & 0x1FFF) << 16);
  }

  pCanMsg->Dlc = CAN->sMsgObj[msgif].MCR & 0x0F;

  pCanMsg->Data[0] = (u8) CAN->sMsgObj[msgif].DA1R;
  pCanMsg->Data[1] = (u8)(CAN->sMsgObj[msgif].DA1R >> 8);
  pCanMsg->Data[2] = (u8) CAN->sMsgObj[msgif].DA2R;
  pCanMsg->Data[3] = (u8)(CAN->sMsgObj[msgif].DA2R >> 8);
  pCanMsg->Data[4] = (u8) CAN->sMsgObj[msgif].DB1R;
  pCanMsg->Data[5] = (u8)(CAN->sMsgObj[msgif].DB1R >> 8);
  pCanMsg->Data[6] = (u8) CAN->sMsgObj[msgif].DB2R;
  pCanMsg->Data[7] = (u8)(CAN->sMsgObj[msgif].DB2R >> 8);

  return SUCCESS;
}

/*******************************************************************************
* Function Name  : CAN_BasicSendMessage                                      
* Description    : Starts transmission of a message in BASIC mode. This mode 
*                  does not use the message RAM. the interface IF0 is used for
*                  message transmission             
* Input          : - pCanMsg: Pointer to the message structure containing data to       
*                  transmit.
*                  - Pointer on a CAN_TypdeDef structure                                                  
* Output         : None                                                      
* Return         : An ErrorStatus enumuration value:
*                         - SUCCESS: Transmission OK
*                         - ERROR: No transmission
*******************************************************************************/
ErrorStatus CAN_BasicSendMessage(CAN_TypeDef *CAN, canmsg* pCanMsg)
{
  /* If a transmission is on going return ERROR */
  if (CAN->sMsgObj[CAN_IF0].CRR == CAN_CRR_BUSY )
    return ERROR;

  CAN->SR &= (u16)(~CAN_SR_TXOK);

  /* Fill the arbitration field */
  if (pCanMsg->IdType == CAN_STD_ID)
  {
    /* standard ID */
    CAN->sMsgObj[CAN_IF0].A1R = 0;
    CAN->sMsgObj[CAN_IF0].A2R = (CAN->sMsgObj[CAN_IF0].A2R & 0xE000) | STD_FIXED_ID_ARB(pCanMsg->Id);
  }
  else
  {
    /* extended ID */
    CAN->sMsgObj[CAN_IF0].A1R = EXT_FIXED_ID_ARB_L(pCanMsg->Id);
    CAN->sMsgObj[CAN_IF0].A2R = ((CAN->sMsgObj[CAN_IF0].A2R) & 0xE000) | CAN_A2R_XTD| EXT_FIXED_ID_ARB_H(pCanMsg->Id);
  }

  CAN->sMsgObj[CAN_IF0].MCR = (CAN->sMsgObj[CAN_IF0].MCR & 0xFCF0) | pCanMsg->Dlc;

  CAN->sMsgObj[CAN_IF0].DA1R = ((u16)pCanMsg->Data[1]<<8) | pCanMsg->Data[0];
  CAN->sMsgObj[CAN_IF0].DA2R = ((u16)pCanMsg->Data[3]<<8) | pCanMsg->Data[2];
  CAN->sMsgObj[CAN_IF0].DB1R = ((u16)pCanMsg->Data[5]<<8) | pCanMsg->Data[4];
  CAN->sMsgObj[CAN_IF0].DB2R = ((u16)pCanMsg->Data[7]<<8) | pCanMsg->Data[6];

   /* request transmission */
   CAN->sMsgObj[CAN_IF0].CRR = CAN_CRR_BUSY;
  return SUCCESS;
} 

/*******************************************************************************
* Function Name  : CAN_BasicReceiveMessage                                   
* Description    : Gets the message in BASIC mode, if received. This mode does
*                  not use the message RAM. the interface IF1 is used for message
*                  reception                
* Input          : - Pointer on a CAN_TypdeDef structure
*                  - pCanMsg: pointer to the message structure where message is copied.    
* Output         : None                                                      
* Return         : An ErrorStatus enumuration value:
*                         - SUCCESS: Reception OK
*                         - ERROR: No message pending
*******************************************************************************/
ErrorStatus CAN_BasicReceiveMessage(CAN_TypeDef *CAN, canmsg* pCanMsg)
{
  if ((CAN->sMsgObj[CAN_IF1].MCR & CAN_MCR_NEWDAT) == 0)
    return ERROR;
  
  CAN->SR &= (u16)(~CAN_SR_RXOK);

  /* Clear the NEWDAT in the message control register */
  CAN->sMsgObj[CAN_IF1].MCR &= (u16)~CAN_MCR_NEWDAT; 

  /* read the arbitration field */
  if ((CAN->sMsgObj[CAN_IF1].A2R & CAN_A2R_XTD) == 0)
  {
    /* standard ID */
    pCanMsg->IdType = CAN_STD_ID;
    pCanMsg->Id = (CAN->sMsgObj[CAN_IF1].A2R >> 2) & 0x07FF;
  }
  else
  {
    /* extended ID */
    pCanMsg->IdType = CAN_EXT_ID;
	pCanMsg->Id  = CAN->sMsgObj[CAN_IF1].A1R | ((u32)(CAN->sMsgObj[CAN_IF1].A2R & 0x1FFF) << 16);
  }

  pCanMsg->Dlc = CAN->sMsgObj[CAN_IF1].MCR & 0x0F;

  pCanMsg->Data[0] = (u8) CAN->sMsgObj[CAN_IF1].DA1R;
  pCanMsg->Data[1] = (u8)(CAN->sMsgObj[CAN_IF1].DA1R >> 8);
  pCanMsg->Data[2] = (u8) CAN->sMsgObj[CAN_IF1].DA2R;
  pCanMsg->Data[3] = (u8)(CAN->sMsgObj[CAN_IF1].DA2R >> 8);
  pCanMsg->Data[4] = (u8) CAN->sMsgObj[CAN_IF1].DB1R;
  pCanMsg->Data[5] = (u8)(CAN->sMsgObj[CAN_IF1].DB1R >> 8);
  pCanMsg->Data[6] = (u8) CAN->sMsgObj[CAN_IF1].DB2R;
  pCanMsg->Data[7] = (u8)(CAN->sMsgObj[CAN_IF1].DB2R >> 8);

  return SUCCESS;
}
 
/*******************************************************************************
* Function Name  : CAN_WaitEndOfTx
* Description    : Wait until current transmission is finished
* Input          : Pointer on a CAN structure CAN1 or CAN 2 
* Output         : None
* Return         : None
*******************************************************************************/
void CAN_WaitEndOfTx( CAN_TypeDef* CAN )
{
  while ((CAN->SR & CAN_SR_TXOK) == 0)
  {
    /*Wait*/
  }
  CAN->SR &= (u16)~CAN_SR_TXOK;
}

/*******************************************************************************
* Function Name  : CAN_GetMsgReceiveStatus                                      
* Description    : Tests if a message object is received in the message object
*                  passed in parameter.             
* Input          : - Pointer on a CAN_TypdeDef structure
*                  - msgobj: specifies the Message object number, from 0 to 31.                       
* Output         : None                                                      
* Return         : SET if  the corresponding message object has  
*                  received a message waiting to be copied, else RESET.           
*******************************************************************************/
FlagStatus CAN_GetMsgReceiveStatus(CAN_TypeDef *CAN, u16 msgobj)
{
   if ( msgobj < 16)
   {
     if ( CAN->ND1R & (1 << msgobj) )
	 return SET;
	 else
	 return RESET;
   }
   else
   {
     if ( CAN->ND2R & (1 << (msgobj-16)) )
      return SET;
     else
      return RESET; 
   }
}

/*******************************************************************************
* Function Name  : CAN_GetMsgTransmitRequestStatus                                   
* Description    : Tests the request status of a transmitted message.          
* Input          :- Pointer on a CAN_TypdeDef structure 
*                 - msgobj: specifies the Message object number, from 0 to 31.                      
* Output         : None                                                      
* Return         : SET if the corresponding message is requested
*                  to transmit, else RESET .                                       
*******************************************************************************/
FlagStatus CAN_GetMsgTransmitRequestStatus(CAN_TypeDef *CAN, u16 msgobj)
{
  if ( msgobj < 16)
   {
     if ( CAN->TXR1R & (1 << msgobj) )
	 return SET;
	 else
	 return RESET;
   }
   else
   {
     if ( CAN->TXR2R & (1 << (msgobj-16)) )
      return SET;
     else
      return RESET; 
   }
}

/*******************************************************************************
* Function Name  : CAN_GetMsgInterruptStatus                                    
* Description    : Tests the interrupt status of a message object.             
* Input          : - Pointer on a CAN_TypdeDef structure
*                  - msgobj: specifies the Message object number, from 0 to 31.                      
* Output         : None                                                      
* Return         : A non-zero value if the corresponding message has an      
*                  interrupt pending, else 0.                                 
*******************************************************************************/
FlagStatus CAN_GetMsgInterruptStatus(CAN_TypeDef *CAN, u16 msgobj)
{
  if ( msgobj < 16)
   {
     if ( CAN->IP1R & (1 << msgobj) )
	 return SET;
	 else
	 return RESET;
   }
   else
   {
     if ( CAN->IP2R & (1 << (msgobj-16)) )
      return SET;
     else
      return RESET; 
   }
}

/*******************************************************************************
* Function Name  : CAN_GetMsgValidStatus                                         
* Description    : Tests the validity of a message object (ready to use).      
* Input          : - Pointer on a CAN_TypdeDef structure
*                  - msgobj: specifies the Message object number, from 0 to 31.                      
* Output         : None                                                      
* Return         : SET if the corresponding message object is   
*                  valid, else RESET.                                             
*******************************************************************************/
FlagStatus CAN_GetMsgValidStatus(CAN_TypeDef *CAN, u16 msgobj)
{
  if ( msgobj < 16)
   {
     if ( CAN->MV1R & (1 << msgobj) )
	 return SET;
	 else
	 return RESET;
   }
   else
   {
     if ( CAN->MV2R & (1 << (msgobj-16)) )
      return SET;
     else
      return RESET; 
   }
}

/*******************************************************************************
* Function Name  : CAN_GetFlagStatus                                         
* Description    : Reads the status of the CAN flags TxOK, RxOK, EPASS, EWARN
*                  and Bus OFF         
* Input          : - Pointer on CAN structure CAN_TypeDef
*                  - One of the following parameters: 
*                      + CAN_SR_TXOK
*                      + CAN_SR_RXOK
*                      + CAN_SR_EPASS
*                      + CAN_SR_EWARN
*                      + CAN_SR_BOFF                      
* Output         : None                                                      
* Return         : SET if the input parameter is set else 0                                           
*******************************************************************************/
FlagStatus CAN_GetFlagStatus ( CAN_TypeDef *CAN, u16 CAN_Flag )
{
	if( CAN->SR & CAN_Flag ) 
	{
		return SET;
	 }
	 return RESET;
}

/*******************************************************************************
* Function Name  : CAN_GetTransmitErrorCounter                                            
* Description    : Reads the CAN cell transmit error counter        
* Input          : Pointer on CAN structure CAN_TypeDef                      
* Output         : None                                                      
* Return         : Transmit Error Counter value between 0..255                                            
*******************************************************************************/
u8 CAN_GetTransmitErrorCounter ( CAN_TypeDef *CAN )
{
 return( CAN->ERR & 0x00FF );
}

/*******************************************************************************
* Function Name  : CAN_GetReceiveErrorCounter                                            
* Description    : Reads the CAN cell receive error counter        
* Input          : Pointer on CAN structure CAN_TypeDef                      
* Output         : None                                                      
* Return         : Receive Error Counter value between 0..127                                            
*******************************************************************************/
u8 CAN_GetReceiveErrorCounter ( CAN_TypeDef *CAN)
{
 return ( ( CAN->ERR & 0x7F00 ) >> 8);
}

/******************* (C) COPYRIGHT 2007 STMicroelectronics *****END OF FILE****/
